#include <cstdio>
#include <vector>
using namespace std;

const int MAXN = 30;
int N, M;
char a[MAXN][MAXN];

bool ok(int r, int c) {
	return r >= 1 && r <= N && c >= 1 && c <= M;
}

bool checkCell(int r1, int c1, int r2, int c2) {
	if (r1 == r2 && c1 == c2)
		return a[r1][c1] == '#';
	if (ok(r2, c2))
		return a[r1][c1] != a[r2][c2];
	return a[r1][c1] == '#';
}

bool checkHorizontal(int k) {
	for (int i = 1; i <= N; ++i) {
		for (int j = 1; j <= M; ++j) {
			int i2 = k + k + 1 - i, j2 = j;
			if (!checkCell(i, j, i2, j2))
				return false;
		}
	}
	return true;
}

bool checkVertical(int k) {
	for (int i = 1; i <= N; ++i) {
		for (int j = 1; j <= M; ++j) {
			int i2 = i, j2 = k + k + 1 - j;
			if (!checkCell(i, j, i2, j2))
				return false;
		}
	}
	return true;
}

bool checkDiagonal1(int k) {
	for (int i = 1; i <= N; ++i) {
		for (int j = 1; j <= M; ++j) {
			int i2 = k - j, j2 = k - i;
			if (!checkCell(i, j, i2, j2))
				return false;
		}
	}
	return true;
}

bool checkDiagonal2(int k) {
	for (int i = 1; i <= N; ++i) {
		for (int j = 1; j <= M; ++j) {
			int i2 = j + k, j2 = i - k;
			if (!checkCell(i, j, i2, j2))
				return false;
		}
	}
	return true;
}

void updateAns(vector<int> &ans, int r1, int c1, int r2, int c2) {
	vector<int> v(4, 0);
	v[0] = r1;
	v[1] = c1;
	v[2] = r2;
	v[3] = c2;
	if (v < ans)
		ans = v;
}

int main() {
	int T;
	scanf("%d", &T);
	while (T--) {
		scanf("%d%d", &N, &M);
		for (int i = 1; i <= N; ++i) {
			scanf(" %s", a[i] + 1);
		}
		vector<int> ans(4, MAXN);
		// Horizontal
		for (int i = 1; i < N; ++i) {
			if (checkHorizontal(i))
				updateAns(ans, i, 1, i, M);
		}
		// Vertical
		for (int i = 1; i < M; ++i) {
			if (checkVertical(i))
				updateAns(ans, 1, i, N, i);
		}
		// Diagonal
		for (int i = 2, r1 = 1, c1 = 1, r2 = 1, c2 = 1; i <= N + M; ++i) { // Fold: r + c = i
			if (checkDiagonal1(i))
				updateAns(ans, r1, c1, r2, c2);
			++r1;
			if (r1 > N)
				--r1, ++c1;
			++c2;
			if (c2 > M)
				--c2, ++r2;
		}
		for (int i = 1 - M, r1 = 1, c1 = M, r2 = 1, c2 = M; i <= N - 1; ++i) { // Fold: r - c = i
			if (checkDiagonal2(i))
				updateAns(ans, r1, c1, r2, c2);
			--c1;
			if (c1 < 1)
				++c1, ++r1;
			++r2;
			if (r2 > N)
				--r2, --c2;
		}
		printf("%d %d %d %d\n", ans[0], ans[1], ans[2], ans[3]);
	}
	return 0;
}
